#!/usr/bin/env python
#coding=utf8
"""
$Id: SQLite.py,v 1.2 2014/05/13 11:09:30 alfille Exp $

Copyright (c) 2014 C-Bell (VAGor). All rights reserved.

Модуль обеспечивает работу с базами данных SQLite:
    -   проверку наличия БД Distiller.db;
    -   проверку наличия таблиц в ней;
    -   заполнение таблиц данными цифровых термометров DS18B20

Для работы программы должны быть установлены пакеты:
    python
"""

import sqlite3, DS18B20, threading, time

db_name='Distiller.db'      #Имя базы данных
db_con=sqlite3.connect(db_name) #открыть БД
db_cur=db_con.cursor()      #создать курсор БД

def matchDS18B20list():
    """Проверка соответствия списка термометров в БД перечню термометров
    на шине 1-wire"""
    db_cur.execute("SELECT ID FROM DS18B20cur")
    #Преобразование списка кортежей в список
    ListDB=[]
    for ts in db_cur:
        tss,=ts
        ListDB.append(tss)
    #Сравнение длина списков
    if len(DS18B20.list()) != len(ListDB):
        return False
    #Проверка идентичности списков
    for item in DS18B20.list():
        if not item in ListDB:
            return False
    return True

def filDS18B20():
    """Заполнение таблицы DS18B20cur в БД списком цифровых термометров"""
    #Очистка таблицы
    db_cur.execute("DELETE FROM DS18B20cur")
    #Запись нового списка термометров DS18B20 в БД
    for ts in DS18B20.list():
        db_cur.execute("INSERT INTO DS18B20cur (ID) values (?)" , (ts,))
    db_con.commit()

def isTable(table_name):
    """Проверяется наличие таблицы с именем table_name в БД и возвращается
    соответствующее логическое значение"""
    return ((table_name,) in db_cur.execute("SELECT name FROM sqlite_master WHERE type='table' AND name=?;" , (table_name,)))

def delTable(table_name):
    """Удаление из БД таблицы с именем table_name"""
    db_cur.execute("DROP TABLE IF EXISTS %s" % table_name)

def createDS18B20():
    """Создание таблицы, хранящей выборки температур DS18B20"""
    __SQL__="CREATE TABLE DS18B20 (DateTime text"
    for ts in DS18B20.list():
        __SQL__+=", '"+ts+"' text"
    __SQL__+=")"
    db_cur.execute(__SQL__)

#Разрешитель измерения температуры
__Measure__=False

def StartMeasure():
    """Запуск процедуры измерения температур и сохранения их в БД"""
    global __Measure__
    __Measure__=True
    __thread__=threading.Thread(target=Measure)
    __thread__.start()

def StopMeasure():
    """Останов процесса измерения температуры"""
    global __Measure__
    __Measure__=False

def Measure():
    """Измерение температур и сохранения их в БД"""
    __con__=sqlite3.connect(db_name) #открыть БД
    __cur__=__con__.cursor()      #создать курсор БД
    while __Measure__:
        __insert__="INSERT INTO DS18B20 (DateTime"
        __values__=" VALUES ('"+time.strftime("%Y.%m.%d %H:%M:%S")+"'"
        #Сохранение значений температур DS18B20 в таблице DS18B20cur
        for ts in DS18B20.temperatures():
            ID,T=ts
            __cur__.execute("UPDATE DS18B20cur SET T=? WHERE ID=?", (T,ID))
            __insert__+=", '"+ID+"'"
            __values__+=", '"+str(T)+"'"
        __insert__+=")"
        __values__+=")"
        #Сохранение значений в таблице DS18B20
        __cur__.execute(__insert__+__values__)
        __con__.commit()

 

def main():
    """Тестовая функция запускает и останавливает процесс заполнения
    таблиц данными цифровых термометров DS18B20"""
    StartMeasure()
    print("Запуск измерения, ожидание 20 сек...")
    time.sleep(20)
    print("Останов измерения.")
    StopMeasure()
    db_cur.execute("SELECT * FROM DS18B20cur")
    for row in db_cur:
        ID, Name, T = row
        print (u"Термометр %s, T=%s" % (ID,T))
    print("")
    db_cur.execute("PRAGMA table_info(DS18B20)")
    columns=[]
    for column in db_cur:
        columns.append(column[1])
    print columns
    db_cur.execute("SELECT * FROM DS18B20")
    for row in db_cur:
        print row

#Проверка наличия таблицы DS18B20cur, создание в случае отсутствия и заполнение списком DS18B20
if not isTable('DS18B20cur'):
    print ('Таблица DS18B20cur в базе данных Distiller.db не обнаружена, создание таблицы')
    db_cur.execute("CREATE TABLE DS18B20cur (ID text,Name text, T real)")
    print ('Таблица DS18B20cur создана, заполнение таблицы списком DS18B20')
    filDS18B20()            #заполнить списком DS18B20
    delTable("DS18B20")     #удалить таблицу хранения показаний DS18B20
    #запустить процедуру определения места термометров
    print ('Cписок DS18B20 сохранен в БД')

#Проверка соответствия списка термометров в БД перечню термометров на шине 1-wire
elif not matchDS18B20list():
    print ("Список DS18B20 не соответствует списку DS18B20 на шине 1-wire, обновление списка")
    filDS18B20()
    print ('Cписок DS18B20 в БД обновлен')
    delTable("DS18B20")     #удалить таблицу хранения показаний DS18B20
    #Тут нужно определить местоположение термометров

#Проверка наличия таблицы DS18B20, создание её копии, создание в случае отсутствия и заполнение списком DS18B20
if isTable('DS18B20'):
        print("Таблица DS18B20 найдена в БД, удаление таблицы DS18B20old")
        delTable("DS18B20old")
        print("Таблица DS18B20old удалена, переименование таблицы DS18B20 в DS18B20old")
        db_cur.execute("ALTER TABLE DS18B20 RENAME TO DS18B20old")
        print("Таблица DS18B20 переименована в DS18B20old, создание новой таблицы DS18B20")
        createDS18B20()
        print("Таблица DS18B20 создана")
else:
        print("Таблица DS18B20 не найдена в БД, создание новой таблицы DS18B20")
        createDS18B20()
        print("Таблица DS18B20 создана")

if __name__=="__main__":
        main()


